import { Connector } from 'laravel-echo/dist/connector';
import { SseChannel } from "../channel/sse-channel";
import { SsePrivateChannel } from "../channel/sse-private-channel";
import { SsePresenceChannel } from "../channel/sse-presence-channel";
let md5 = require('md5');
/**
 * This class creates a null connector.
 */
export class SseConnector extends Connector {
    constructor() {
        super(...arguments);
        /**
         * All of the subscribed channel names.
         */
        this.channels = {};
    }
    /**
     * Create a fresh connection.
     */
    connect() {
        if (typeof this.options.client !== 'undefined') {
            this.eventSource = this.options.client;
        }
        else {
            this.eventSource = new EventSource(this.options.host + this.options.path, this.options);
        }
        this.eventSourceId = md5((new Date()).toTimeString());
    }
    /**
     * Listen for an event on a channel instance.
     */
    listen(name, event, callback) {
        return this.channel(name).listen(event, callback);
    }
    /**
     * Get a channel instance by name.
     */
    channel(channel) {
        if (!this.channels[name]) {
            this.channels[name] = new SseChannel(this.eventSource, name, this.options);
        }
        return this.channels[name];
    }
    /**
     * Get a private channel instance by name.
     */
    privateChannel(channel) {
        if (!this.channels['private-' + name]) {
            this.channels['private-' + name] = new SsePrivateChannel(this.eventSource, 'private-' + name, this.options);
        }
        return this.channels['private-' + name];
    }
    /**
     * Get a presence channel instance by name.
     */
    presenceChannel(channel) {
        if (!this.channels['presence-' + name]) {
            this.channels['presence-' + name] = new SsePresenceChannel(this.eventSource, 'presence-' + name, this.options);
        }
        return this.channels['presence-' + name];
    }
    /**
     * Leave the given channel, as well as its private and presence variants.
     */
    leave(channel) {
        //
    }
    /**
     * Leave the given channel.
     */
    leaveChannel(channel) {
        //
    }
    /**
     * Get the socket ID for the connection.
     */
    socketId() {
        return this.eventSourceId;
    }
    /**
     * Disconnect the connection.
     */
    disconnect() {
        if (typeof this.eventSource !== 'undefined') {
            this.eventSource.close();
        }
    }
}
