import {EventFormatter} from "laravel-echo/dist/util/event-formatter";
import {Channel} from "laravel-echo/dist/channel/channel";

/**
 * This class represents a null channel.
 */
export class SseChannel extends Channel {

    /**
     * The eventsource instance
     */
    eventSource: EventSource;

    /**
     * The name of the channel.
     */
    name: any;

    /**
     * Channel options.
     */
    options: any;

    /**
     * The event formatter.
     */
    eventFormatter: EventFormatter;

    /**
     * The event callbacks applied to the channel.
     */
    events: any = {};

    /**
     * Create a new class instance.
     */
    constructor(eventSource: EventSource, name: any, options: any) {
        super();

        this.name = name;
        this.options = options;
        this.eventSource = eventSource;
        this.eventFormatter = new EventFormatter(this.options.namespace);

    }

    /**
     * Listen for an event on the channel instance.
     */
    listen(event: string, callback: Function): SseChannel {
        this.events[event] = callback;
        this.eventSource.addEventListener(event, callback());
        return this;
    }

    /**
     * Stop listening for an event on the channel instance.
     */
    stopListening(event: string): SseChannel {
        this.eventSource.removeEventListener(event, this.events[event]);
        return this;
    }

}
